@@ -673,7 +673,7 @@ module PhusionPassenger
         # macOS >= 10.12 Sierra no longer includes apr-config
         # so we'll use a hardcoded path here.
         if os_name_simple == 'macosx' && os_version >= '10.12' && httpd == '/usr/sbin/httpd'
-          apxs2_flags << ' -I/usr/include/apr-1'
+          apxs2_flags << " -I#{`xcrun --show-sdk-path`.strip}/usr/include/apr-1"
         end
 
         apxs2_flags.strip!
@@ -692,7 +692,7 @@ module PhusionPassenger
         #   /usr/sbin/httpd (for architecture ppc64):       Mach-O 64-bit executable ppc64
         #   /usr/sbin/httpd (for architecture i386):        Mach-O executable i386
         #   /usr/sbin/httpd (for architecture x86_64):      Mach-O 64-bit executable x86_64
-        #   /usr/sbin/httpd (for architecture arm64e):	    Mach-O 64-bit executable arm64e
+        #   /usr/sbin/httpd (for architecture arm64e):	    Mach-O 64-bit executable arm64
         #
         # But on some machines, it may output just:
         #
@@ -701,12 +701,14 @@ module PhusionPassenger
         # (http://code.google.com/p/phusion-passenger/issues/detail?id=236)
         output = `file "#{httpd}"`.strip
         if output =~ /Mach-O fat file/ && output !~ /for architecture/
-          architectures = ["i386", "ppc", "x86_64", "ppc64", "arm64e"]
+          architectures = ["i386", "ppc", "x86_64", "ppc64", "arm64"]
         else
           architectures = []
           output.split("\n").grep(/for architecture/).each do |line|
             line =~ /for architecture (.*?)\)/
-            architectures << $1
+            arch = $1
+            arch = "arm64" if arch.start_with?("arm64")
+            architectures << arch
           end
         end
         # The compiler may not support all architectures in the binary.
@@ -838,9 +840,7 @@ module PhusionPassenger
             # files. On Hongli's and Camden's systems it's inside the Xcode directory,
             # while on https://github.com/phusion/passenger/issues/1986 it's inside
             # /Library/Developer/CommandLineTools.
-            xcode_prefix = `/usr/bin/xcode-select -p`.strip
-            ["-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apr-1 " \
-             "-I#{xcode_prefix}/SDKs/MacOSX.sdk/usr/include/apr-1",
+            ["-I#{`xcrun --show-sdk-path`.strip}/usr/include/apr-1",
              '-lapr-1']
           else
             ['-I/usr/include/apr-1', '-lapr-1']
@@ -888,9 +888,7 @@ module PhusionPassenger
           if os_version >= '10.13'
             # On macOS >= 10.13 High Sierra /usr/include no longer
             # exists.
-            xcode_prefix = `/usr/bin/xcode-select -p`.strip
-            ["-I#{xcode_prefix}/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk/usr/include/apr-1 " \
-             "-I#{xcode_prefix}/SDKs/MacOSX.sdk/usr/include/apr-1",
+            ["-I#{`xcrun --show-sdk-path`.strip}/usr/include/apr-1",
              '-laprutil-1']
           else
             ['-I/usr/include/apr-1', '-laprutil-1']
