@@ -687,7 +687,7 @@ namespace tut {
 			"Host: foo\r\n\r\n");
 		string response = readAll(fd, 1024).first;
 		ensure(containsSubstring(response,
-			"HTTP/1.1 505 HTTP Version Not Supported\r\n"
+			"HTTP/1.0 505 HTTP Version Not Supported\r\n"
 			"Status: 505 HTTP Version Not Supported\r\n"
 			"Content-Type: text/html; charset=UTF-8\r\n"));
 		ensure(containsSubstring(response,
@@ -733,16 +733,23 @@ namespace tut {
 			"0\r\n"
 			"\r\n");
 		string response = readAll(fd, 1024).first;
-		ensure(containsSubstring(response,
+		ensure("Response starts with error",
+			startsWith(response,
 			"HTTP/1.0 400 Bad Request\r\n"
 			"Status: 400 Bad Request\r\n"
 			"Content-Type: text/html; charset=UTF-8\r\n"));
-		ensure(containsSubstring(response,
-			"Connection: close\r\n"
-			"Content-Length: 79\r\n"
-			"cache-control: no-cache, no-store, must-revalidate\r\n"
-			"\r\n"
-			"Bad request (request may not contain both Content-Length and Transfer-Encoding)"));
+		// Not sure why the error is HPE_INVALID_CONTENT_LENGTH
+		// instead of HPE_UNEXPECTED_CONTENT_LENGTH. But we already have
+		// a different test that tests Transfer-Encoding and Content-Length
+		// simultaneously are not allowed, so this is not a big deal.
+		ensure("Response ends with error",
+			endsWith(response,
+				"Connection: close\r\n"
+				"Content-Length: 42\r\n"
+				"cache-control: no-cache, no-store, must-revalidate\r\n"
+				"\r\n"
+				"invalid character in content-length header"));
+		ensure_equals("Response size is correct", response.size(), 265u);
 	}
 
 	TEST_METHOD(17) {
@@ -759,16 +766,44 @@ namespace tut {
 			"0\r\n"
 			"\r\n");
 		string response = readAll(fd, 1024).first;
-		ensure(containsSubstring(response,
+		ensure("Response starts with error",
+			startsWith(response,
 			"HTTP/1.0 400 Bad Request\r\n"
 			"Status: 400 Bad Request\r\n"
 			"Content-Type: text/html; charset=UTF-8\r\n"));
-		ensure(containsSubstring(response,
+		ensure("Response ends with error",
+			endsWith(response,
 			"Connection: close\r\n"
 			"Content-Length: 27\r\n"
 			"cache-control: no-cache, no-store, must-revalidate\r\n"
 			"\r\n"
 			"invalid character in header"));
+		ensure_equals("Response size is correct", response.size(), 250u);
+	}
+
+	TEST_METHOD(18) {
+		set_test_name("Request Smuggling type: 3");
+
+		connectToServer();
+		sendRequest(
+			"POST / HTTP/1.1\r\n"
+			"Host: whatever\r\n"
+			"Transfer-Encoding: ,chunked\r\n"
+			"\r\n"
+			"0\r\n"
+			"\r\n"
+			);
+		string response = readAll(fd, 1024).first;
+		ensure_equals("Response is ok",
+			response,
+			"HTTP/1.1 200 OK\r\n"
+			"Status: 200 OK\r\n"
+			"Content-Type: text/plain\r\n"
+			"Date: Thu, 11 Sep 2014 12:54:09 GMT\r\n"
+			"Connection: close\r\n"
+			"Content-Length: 7\r\n"
+			"\r\n"
+			"hello /");
 	}
 
 	/***** Fixed body handling *****/
