@@ -1,5 +1,6 @@
 require 'erb'
 require 'fileutils'
+require 'time'
 PhusionPassenger.require_passenger_lib 'platform_info/apache'
 PhusionPassenger.require_passenger_lib 'platform_info/ruby'
 
@@ -94,10 +95,9 @@ class Apache2Controller
 
     if @codesigning_identity
       require 'open3'
-      Open3.capture3("codesign", "-s", @codesigning_identity, "--keychain", File.expand_path("~/Library/Keychains/login.keychain-db"), @mod_passenger) do |stdout, stderr, status|
-        if !status.success?
-          raise "Could not sign Apache module at #{@mod_passenger} with authority #{@codesigning_identity}: #{stderr}"
-        end
+      stdout, stderr, status = Open3.capture3("codesign", "--force", "-s", @codesigning_identity, "--keychain", File.expand_path(@codesigning_keychain), @mod_passenger)
+      if !status.success?
+        raise "Could not sign Apache module at #{@mod_passenger} with authority #{@codesigning_identity}: #{stderr}"
       end
     end
 
@@ -114,19 +114,40 @@ class Apache2Controller
         '--vgdb-error=1', '--trace-children=no'] + command
     end
 
+    prev_error_log_position = error_log_position
     if !system(*command)
-      raise "Could not start an Apache server."
+      raise [
+        "Could not start an Apache server: #{$?}",
+        "\t---------------- Begin logs -------------------",
+        read_error_log_starting_from(prev_error_log_position).split("\n").map{ |l| "\t#{l}" }.join("\n"),
+        "\t---------------- End logs -------------------",
+      ].join("\n")
     end
 
+    # Wait until the PID file has been created.
+    wait_start_time = Time.now
     begin
-      # Wait until the PID file has been created.
       Timeout::timeout(20) do
         while !File.exist?("#{@server_root}/httpd.pid")
           sleep(0.1)
         end
       end
-      # Wait until Apache is listening on the server port.
-      Timeout::timeout(7) do
+    rescue Timeout::Error
+      end_start_time = Time.now
+      raise [
+        "Timeout waiting for an Apache server report its PID:",
+        "\t---------------- Begin logs -------------------",
+        read_error_log_starting_from(prev_error_log_position).split("\n").map{ |l| "\t#{l}" }.join("\n"),
+        "\t---------------- End logs -------------------",
+        "Started waiting at #{wait_start_time.iso8601(3)}",
+        "  Ended waiting at #{end_start_time.iso8601(3)}",
+      ].join("\n")
+    end
+
+    # Wait until Apache is listening on the server port.
+    wait_start_time = Time.now
+    begin
+      Timeout::timeout(30) do
         done = false
         while !done
           begin
@@ -139,8 +160,17 @@ class Apache2Controller
         end
       end
     rescue Timeout::Error
-      raise "Could not start an Apache server."
+      end_start_time = Time.now
+      raise [
+        "Timeout waiting for an Apache server to listen on port #{@port}:",
+        "\t---------------- Begin logs -------------------",
+        read_error_log_starting_from(prev_error_log_position).split("\n").map{ |l| "\t#{l}" }.join("\n"),
+        "\t---------------- End logs -------------------",
+        "Started waiting at #{wait_start_time.iso8601(3)}",
+        "  Ended waiting at #{end_start_time.iso8601(3)}",
+      ].join("\n")
     end
+
     Dir["#{@server_root}/*"].each do |filename|
       if File.file?(filename)
         File.chmod(0666, filename)
@@ -249,6 +279,26 @@ private
     end
   end
 
+  def error_log_position
+    if @log_file
+      File.open(@log_file, 'rb') do |f|
+        f.seek(0, IO::SEEK_END)
+        f.pos
+      end
+    end
+  rescue Errno::ENOENT
+    nil
+  end
+
+  def read_error_log_starting_from(prev_pos)
+    File.open(@log_file, 'r:utf-8') do |f|
+      f.seek(prev_pos || 0, IO::SEEK_SET)
+      f.read
+    end
+  rescue Errno::ENOENT
+    "(no log file)"
+  end
+
   def modules_dir
     PlatformInfo.apache2_modulesdir
   end
