@@ -815,6 +815,52 @@ void ensure_equals(const Q& actual, const T& expected)
     ensure_equals<>(0, actual, expected);
 }
 
+/**
+ * Tests two objects for not being equal.
+ * Throws if equal.
+ *
+ * NB: T must have operator << defined somewhere, or
+ * client code will not compile at all!
+ */
+template <class T, class Q>
+void ensure_not_equals(const char* msg, const Q& a, const T& b)
+{
+    if (a == b)
+    {
+        std::stringstream ss;
+        ss << (msg ? msg : "")
+            << (msg ? ":" : "")
+            << " expected not to be '"
+            << a
+            << "'";
+        throw failure(ss.str().c_str());
+    }
+}
+
+/**
+ * Tests that A > B.
+ * Throws if false.
+ *
+ * NB: both T and Q must have operator << defined somewhere, or
+ * client code will not compile at all!
+ */
+template <class T, class Q>
+void ensure_gt(const char* msg, const Q& a, const T& b)
+{
+    if (!(a > b))
+    {
+        std::stringstream ss;
+        ss << (msg ? msg : "")
+            << (msg ? ":" : "")
+            << " expected '"
+            << a
+            << "' to be greater than '"
+            << b
+            << '\'';
+        throw failure(ss.str().c_str());
+    }
+}
+
 /**
  * Tests two objects for being at most in given distance one from another.
  * Borders are excluded.
